
from onlinemid.utils.online_data_structures import NpVec
import numpy as np
def make_onlineroi_from_roi_array(roi_array):
    '''
    input is a roi matrix of the shape
    e.g.  (roi_nr x slow scan dim x fast scan dim)
    e.g.  (5, 512, 128)
    e.g.  (5, 1024)
    where each ( x, 512, 128) is True or False, depending on the roi
    

    returns a dict:
     { "online_roi", online_roi, an array that has shape[0] = number of rois
             for each  compressed_roi = online_roi[n,:]  there are pairs or uint64 that contain respecitvley (index, number of entris) 
       "datasize", the size of the data for which this online roi can be used}

    compressing [[0. 0. 1. 1. 1. 0. 0. 1. 1. 0.]
                 [0. 1. 1. 1. 1. 1. 1. 1. 1. 0.]
                 [1. 1. 0. 0. 0. 0. 0. 0. 0. 1.]]
    
    yields
    online_roi = [[2 3 7 2]
                    [1 8 0 0]
                    [0 2 9 1]]
    data size = 10

    '''
    if roi_array.ndim < 2:
        print("Error in make_onlineroi_from_roi_array, array must have at least 2 dimensions")
    num_rois = roi_array.shape[0]

    def compress_roi_array(roi_array):
        
        compressed_data = NpVec(dtype=np.uint32)
        roi_ravel = roi_array.reshape(-1)
        index = 0
        while index < roi_array.size:
            length = 0

            val = roi_ravel[index]

            if val == True:
                length = 1
                if index+length < roi_array.size:
                    while roi_ravel[index+length] == True:
                        length +=1         

                compressed_data.add(np.array([index, length], dtype=np.uint32))
                index += length
            else:
                index += 1
        return compressed_data.get()
            
    online_roi_data = dict()
    for roi_nr in range(num_rois):
        this_roi_array = roi_array[roi_nr]
        
        online_roi_data[f'{roi_nr}'] = compress_roi_array(this_roi_array)

    # reshape this online_roi_data into a np array.
    online_roi_data_max_length = 2
    for key, this_online_roi_data in online_roi_data.items():
        this_size = this_online_roi_data.size
        if this_size > online_roi_data_max_length:
            online_roi_data_max_length = this_size
    
    online_roi = np.zeros(shape=(num_rois, online_roi_data_max_length), dtype=np.uint32)
    for key, this_online_roi_data in online_roi_data.items():
        this_size = this_online_roi_data.size
        online_roi[int(key),:this_size] = this_online_roi_data


    return { "online_roi", online_roi, "roi_datasize", roi_array[0].size}



