import numpy as np
cimport numpy as cnp  # for cnp.ndarray
from libcpp cimport bool

# Numpy must be initialized. When using numpy from C or Cython you must
# _always_ do that, or you will have segfaults
cnp.import_array()

from numpy cimport ndarray

cdef extern from "<stdint.h>":
    ctypedef signed char int8_t
    ctypedef unsigned char uint8_t
    ctypedef unsigned short uint16_t
    ctypedef signed short int16_t
    ctypedef signed long int int64_t
    ctypedef unsigned long long uint64_t
    ctypedef signed int int32_t
    ctypedef unsigned int uint32_t
    
cdef extern from "numpy/arrayobject.h":
    void PyArray_ENABLEFLAGS(cnp.ndarray arr, int flags)

cdef extern from "runningstats_impl.h":

    void running_stats(
        float*&,
        uint32_t,
        double*&,
        float*&)

    void running_stats_simd(
        double*&,
        uint32_t,
        double*&,
        uint32_t*&,
        float*&,
        uint32_t*&,
        uint32_t)

    void running_stats_simd_float(
        float*&,
        uint32_t,
        float*&,
        uint32_t*&,
        float*&,
        uint32_t*&,
        uint32_t)


    void stats_with_roi(
        float*&,
        uint32_t,
        uint32_t,
        uint32_t*&,
        uint32_t,
        uint32_t,
        int16_t*&,
        uint16_t,
        double*&,
        double*&);


    

cpdef compute_stats(
    cnp.ndarray[cnp.float32_t, ndim=1] data,
    cnp.ndarray[cnp.float64_t, ndim=1] running_mean_mem,
    cnp.ndarray[cnp.float32_t, ndim=1] nanmean_of_data):
    
    running_stats(
        &data[0],
        data.size,
        &running_mean_mem[0],
        &nanmean_of_data[0])

cpdef compute_stats_simd(
    cnp.ndarray[cnp.float64_t, ndim=1] data,
    cnp.ndarray[cnp.float64_t, ndim=1] running_mean_mem,
    cnp.ndarray[cnp.float32_t, ndim=1] nanmean_of_data,
    cnp.ndarray[cnp.uint32_t, ndim=1] simd_vec_size,
    cnp.ndarray[cnp.uint32_t, ndim=1] simd_offset,
    uint32_t num_threads):
    
    running_stats_simd(
        &data[0],
        data.size,
        &running_mean_mem[0],
        &simd_offset[0],
        &nanmean_of_data[0],
        &simd_vec_size[0],
        num_threads)

cpdef compute_stats_simd_float(
    cnp.ndarray[cnp.float32_t, ndim=1] data,
    cnp.ndarray[cnp.float32_t, ndim=1] running_mean_mem,
    cnp.ndarray[cnp.float32_t, ndim=1] nanmean_of_data,
    cnp.ndarray[cnp.uint32_t, ndim=1] simd_vec_size,
    cnp.ndarray[cnp.uint32_t, ndim=1] simd_offset,
    uint32_t num_threads):
    
    running_stats_simd_float(
        &data[0],
        data.size,
        &running_mean_mem[0],
        &simd_offset[0],
        &nanmean_of_data[0],
        &simd_vec_size[0],
        num_threads)

        
cpdef call_roi_stats(
    cnp.ndarray[cnp.float32_t, ndim=1] data, # data
    uint32_t image_size, # image size
    uint32_t num_images, # num images
    cnp.ndarray[cnp.uint32_t, ndim=2] online_roi, # 
    uint32_t num_rois,
    uint32_t online_roi_length_per_roi,
    cnp.ndarray[cnp.int16_t, ndim=1] litframes,
    uint16_t num_litframes,
    cnp.ndarray[cnp.float64_t, ndim=1] memory_for_bunch_resolved_roi_stats,
    cnp.ndarray[cnp.float64_t, ndim=1] memory_for_pixel_stats
     
    ):

    stats_with_roi(
        &data[0],
        image_size,
        num_images,
        &online_roi[0],
        num_rois,
        online_roi_length_per_roi,
        &litframes[0],
        num_litframes,
        &memory_for_bunch_resolved_roi_stats[0],
        &memory_for_pixel_stats[0]
        )





