
#include <cstdint>
#include <memory>
#include <iostream>

void running_stats(
    float* data,
    std::uint32_t data_size,
    double* running_mean_mem,
    float* nanmean_of_data);

void running_stats_simd(
    double* data,
    std::uint32_t data_size,
    double* running_mean_mem,
    std::uint32_t* simd_offset,
    float* nanmean_of_data,
    std::uint32_t* vec_size,
    std::uint32_t numThreads);

void running_stats_simd_float(
    float* data,
    std::uint32_t data_size,
    float* running_mean_mem,
    std::uint32_t* simd_offset,
    float* nanmean_of_data,
    std::uint32_t* vec_size,
    std::uint32_t numThreads);

void stats_with_roi(
    float* data,
    std::uint32_t image_size,
    std::uint32_t num_images,
    std::uint32_t* roi,
    std::uint32_t num_rois,
    std::uint32_t online_roi_length_per_roi,
    std::int16_t* litframes,
    std::uint16_t num_litframes,
    double* memory_mean,
    double* image_memory_mean);


// C++20 make_unique_for_overwrite
template<class T>
    requires (!std::is_array_v<T>)
std::shared_ptr<T> make_shared_for_overwrite()
{
    return std::shared_ptr<T>(new T);
}
 
template<class T>
    requires std::is_unbounded_array_v<T>
std::shared_ptr<T> make_shared_for_overwrite(std::size_t n)
{
    return std::shared_ptr<T>(new std::remove_extent_t<T>[n]);
}
 
template<class T, class... Args>
    requires std::is_bounded_array_v<T>
void make_shared_for_overwrite(Args&&...) = delete;

   